Attribute VB_Name = "modPosition"
'**************************************************************
'**modPosition
'**(c) 1999, Hewlett-Packard Corporation. All rights reserved.
'**
'**This module contains positioning functions for the
'**  CPSS troubleshooting wizard.
'**
'**For public variables and constants, and initialization functions
'**  specific to the troubleshooter, see modInit.
'**For interface element event code, see frmWizard.
'**For other functions specific to the troubleshooter, see modWizard.
'**
'**************************************************************

Option Explicit

'**************************************************************
'Global variables and structures for subroutines
Type RECT
    rectLeft As Long
    rectTop As Long
    rectRight As Long
    rectBottom As Long
End Type

'**Window attribute commands*********************************
Declare Function GetWindowLong Lib "User32" Alias "GetWindowLongA" (ByVal wHwnd As Long, ByVal nIndex As Long) As Long
Declare Function SetWindowLong Lib "User32" Alias "SetWindowLongA" (ByVal wHwnd As Long, ByVal nIndex As Long, ByVal dwNewLong As Long) As Long

'**Window field offsets for GetWindowLong() and GetWindowWord()
Public Const GWL_WNDPROC = (-4)
Public Const GWW_HINSTANCE = (-6)
Public Const GWW_HWNDPARENT = (-8)
Public Const GWW_ID = (-12)
Public Const GWL_STYLE = (-16)
Public Const GWL_EXSTYLE = (-20)

'**Window Styles*********************************************
Public Const WS_OVERLAPPED = &H0&
Public Const WS_POPUP = &H80000000
Public Const WS_CHILD = &H40000000
Public Const WS_MINIMIZE = &H20000000
Public Const WS_VISIBLE = &H10000000
Public Const WS_DISABLED = &H8000000
Public Const WS_CLIPSIBLINGS = &H4000000
Public Const WS_CLIPCHILDREN = &H2000000
Public Const WS_MAXIMIZE = &H1000000
Public Const WS_CAPTION = &HC00000      '  WS_BORDER Or WS_DLGFRAME
Public Const WS_BORDER = &H800000
Public Const WS_DLGFRAME = &H400000
Public Const WS_VSCROLL = &H200000
Public Const WS_HSCROLL = &H100000
Public Const WS_SYSMENU = &H80000
Public Const WS_THICKFRAME = &H40000
Public Const WS_GROUP = &H20000
Public Const WS_TABSTOP = &H10000
Public Const WS_MINIMIZEBOX = &H20000
Public Const WS_MAXIMIZEBOX = &H10000

'**Window style specification constants***********************
Public Const MODE_MINIMIZE = 1
Public Const MODE_MAXIMIZE = 2
Public Const MODE_NORMAL = 4
Public Const MODE_HIDE = 8
Public Const MODE_NOMAXIMIZE = 16
Public Const MODE_NOMINIMIZE = 32
Public Const MODE_NORESIZE = 64
Public Const MODE_NOSYSMENU = 128
Public Const MODE_TOPMOST = 256

'**RedrawWindow function**************************************
Declare Function RedrawWindow Lib "User32" (ByVal wHwnd As Long, lprcUpdate As Any, hrgnUpdate As Long, ByVal fuRedraw As Long) As Long

'**RedrawWindow constants*************************************
Public Const RDW_INVALIDATE = &H1
Public Const RDW_INTERNALPAINT = &H2
Public Const RDW_ERASE = &H4
Public Const RDW_VALIDATE = &H8
Public Const RDW_NOINTERNALPAINT = &H10
Public Const RDW_NOERASE = &H20
Public Const RDW_NOCHILDREN = &H40
Public Const RDW_ALLCHILDREN = &H80
Public Const RDW_UPDATENOW = &H100
Public Const RDW_ERASENOW = &H200
Public Const RDW_FRAME = &H400
Public Const RDW_NOFRAME = &H800

'Resizing constants
Public optionKeepOnTop As Integer
Public Const RS_RESIZE = 1
Public Const RS_REPOSITION = 2
Public Const RS_CHANGEKEEPONTOP = 4
Public Const RS_SIZEPOSITION = 3

Public Const RS_SHOW = -1
Public Const RS_NOSHOW = 0
Public Const RS_SHOWMODAL = 1

Public Const RS_ONTOP = -1
Public Const RS_NOTONTOP = 0

'Positioning constants for cursor positioning
Public Const FIXEDOFFSET = 0
Public Const TOPLEFT = 1
Public Const MIDDLELEFT = 2
Public Const BOTTOMLEFT = 3
Public Const TOPCENTER = 4
Public Const CENTER = 5
Public Const BOTTOMCENTER = 6
Public Const TOPRIGHT = 7
Public Const MIDDLERIGHT = 8
Public Const BOTTOMRIGHT = 9
Public Const PERCENTOFFSET = 10

'********************************************************
'Get the cursor position
Declare Function GetCursorPos Lib "User32" (lpPoint As POINTAPI) As Long
Declare Function SetCursorPos Lib "User32" (ByVal X As Long, ByVal Y As Long) As Long
Declare Function WindowFromPoint Lib "User32" (ByVal xPoint As Long, ByVal yPoint As Long) As Long

'********************************************************
'Get system metric information
'
'Place the following code in the Form_Load event.
'The following code calculates the scale factor required
'to convert system units into your application units.
'Multiply system units by this scale factor to get
'your application units.
'
'XScale& = Screen.Width / GetSystemMetrics(SM_CXSCREEN)
'YScale& = Screen.Height / GetSystemMetrics(SM_CYSCREEN)
Declare Function GetSystemMetrics Lib "User32" (ByVal nIndex As Long) As Long

' GetSystemMetrics() codes
Public Const SM_CXSCREEN = 0
Public Const SM_CYSCREEN = 1
Public Const SM_CXVSCROLL = 2
Public Const SM_CYHSCROLL = 3
Public Const SM_CYCAPTION = 4
Public Const SM_CXBORDER = 5
Public Const SM_CYBORDER = 6
Public Const SM_CXDLGFRAME = 7
Public Const SM_CYDLGFRAME = 8
Public Const SM_CYVTHUMB = 9
Public Const SM_CXHTHUMB = 10
Public Const SM_CXICON = 11
Public Const SM_CYICON = 12
Public Const SM_CXCURSOR = 13
Public Const SM_CYCURSOR = 14
Public Const SM_CYMENU = 15
Public Const SM_CXFULLSCREEN = 16
Public Const SM_CYFULLSCREEN = 17
Public Const SM_CYKANJIWINDOW = 18
Public Const SM_MOUSEPRESENT = 19
Public Const SM_CYVSCROLL = 20
Public Const SM_CXHSCROLL = 21
Public Const SM_DEBUG = 22
Public Const SM_SWAPBUTTON = 23
Public Const SM_RESERVED1 = 24
Public Const SM_RESERVED2 = 25
Public Const SM_RESERVED3 = 26
Public Const SM_RESERVED4 = 27
Public Const SM_CXMIN = 28
Public Const SM_CYMIN = 29
Public Const SM_CXSIZE = 30
Public Const SM_CYSIZE = 31
Public Const SM_CXFRAME = 32
Public Const SM_CYFRAME = 33
Public Const SM_CXMINTRACK = 34
Public Const SM_CYMINTRACK = 35
Public Const SM_CMETRICS = 36

'Get window placement information
'
'This API call lets you determine the normal, maximized,
'and iconized size & position of a window regardless
'of its current window state. The Set function lets you
'specify these values when loading the form.
'
'The following code in your app retrieves the
'WindowPlacement structure. Don't forget to convert these
'system units to your app units.
'
'Dim handle As Long
'Dim successl as long
'handle = Form1.hWnd
'WindowPlace.wLength = 22
'successl = GetWindowPlacement(ByVal handle, WindowPlace)
Type WINDOWPLACEMENT
    wLength As Long 'must be set to 44 before trying to get the structure
    wFlags As Long 'Used only in SetWindowPlacement
    wShowCmd As Long 'Used only in SetWindowPlacement
    iconLeft As Long
    iconTop As Long
    maxLeft As Long
    maxTop As Long
    normalLeft As Long
    normalTop As Long
    normalRight As Long
    normalBottom As Long
End Type

Declare Function FindWindow Lib "User32" Alias "FindWindowA" (lpClassName As Any, lpWindowName As Any) As Long
Declare Function CloseWindow Lib "User32" (ByVal wHwnd As Long) As Long
Declare Function DestroyWindow Lib "User32" (ByVal wHwnd As Long) As Long
Declare Function GetWindowPlacement Lib "User32" (ByVal wHwnd As Long, lpwndpl As WINDOWPLACEMENT) As Long
Declare Function SetWindowPlacement Lib "User32" (ByVal wHwnd As Long, lpwndpl As WINDOWPLACEMENT) As Long
Declare Function ShowWindow Lib "User32" (ByVal wHwnd As Long, ByVal nCmdShow As Long) As Long
Declare Sub MoveWindow Lib "User32" (ByVal wHwnd As Long, ByVal X As Integer, ByVal Y As Integer, ByVal nWidth As Integer, ByVal nHeight As Integer, ByVal bRepaint As Integer)
Declare Function IsWindowVisible Lib "User32" (ByVal wHwnd As Long) As Long
Declare Function GetForegroundWindow Lib "User32" () As Long
Declare Function BringWindowToTop Lib "User32" (ByVal wHwnd As Long) As Long
Declare Function GetActiveWindow Lib "User32" () As Long
Declare Function GetWindow Lib "User32" (ByVal hwnd As Long, ByVal wCmd As Long) As Long
Declare Function GetNextWindow Lib "User32" Alias "GetWindow" (ByVal hwnd As Long, ByVal wFlag As Long) As Long

'**GetWindow constants***************************************************
Public Const GW_CHILD = 5
Public Const GW_HWNDFIRST = 0
Public Const GW_HWNDLAST = 1
Public Const GW_HWNDNEXT = 2
Public Const GW_HWNDPREV = 3
Public Const GW_MAX = 5
Public Const GW_OWNER = 4

'**ShowWindow() constants*************************************************
Public Const SW_HIDE = 0
Public Const SW_SHOWNORMAL = 1
Public Const SW_NORMAL = 1
Public Const SW_SHOWMINIMIZED = 2
Public Const SW_SHOWMAXIMIZED = 3
Public Const SW_MAXIMIZE = 3
Public Const SW_SHOWNOACTIVATE = 4
Public Const SW_SHOW = 5
Public Const SW_MINIMIZE = 6
Public Const SW_SHOWMINNOACTIVE = 7
Public Const SW_SHOWNA = 8
Public Const SW_RESTORE = 9

'Set window position
'
'This API call lets you set the window position, and
'make it stay on top if required.
Declare Function SetWindowPos Lib "User32" (ByVal wHwnd As Long, ByVal wHwndInsertAfter As Long, ByVal X As Long, ByVal Y As Long, ByVal cx As Long, ByVal cy As Long, ByVal wFlags As Long) As Long

'**SetWindowPos Flags
Public Const SWP_NOSIZE = &H1
Public Const SWP_NOMOVE = &H2
Public Const SWP_NOZORDER = &H4
Public Const SWP_NOREDRAW = &H8
Public Const SWP_NOACTIVATE = &H10
Public Const SWP_DRAWFRAME = &H20
Public Const SWP_SHOWWINDOW = &H40
Public Const SWP_HIDEWINDOW = &H80
Public Const SWP_NOCOPYBITS = &H100
Public Const SWP_NOREPOSITION = &H200

'**Window messaging
Declare Function SendMessage Lib "User32" Alias "SendMessageA" (ByVal wHwnd As Long, ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long
Declare Function PostMessage Lib "User32" Alias "PostMessageA" (ByVal wHwnd As Long, ByVal wMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long

'**PostMessage(WM_COMMAND, WM_CLOSE, 0, 0)
Public Const WM_COMMAND = &H111
Public Const WM_CLOSE = &H10

'**SetWindowPos() hWndInsertAfter values
Public Const HWND_TOP = 0
Public Const HWND_BOTTOM = 1
Public Const HWND_TOPMOST = -1
Public Const HWND_NOTOPMOST = -2

'**Get system parameters****************************************
Declare Function SystemParametersInfo Lib "User32" Alias "SystemParametersInfoA" (ByVal uAction As Long, ByVal uParam As Long, lpvParam As Any, ByVal fuWinIni As Long) As Long

Public Const SPI_GETACCESSTIMEOUT = 60
Public Const SPI_GETANIMATION = 72
Public Const SPI_GETBEEP = 1
Public Const SPI_GETBORDER = 5
Public Const SPI_GETDEFAULTINPUTLANG = 89
Public Const SPI_GETDRAGFULLWINDOWS = 38
Public Const SPI_GETFASTTASKSWITCH = 35
Public Const SPI_GETFILTERKEYS = 50
Public Const SPI_GETFONTSMOOTHING = 74
Public Const SPI_GETGRIDGRANULARITY = 18
Public Const SPI_GETHIGHCONTRAST = 66
Public Const SPI_GETICONMETRICS = 45
Public Const SPI_GETICONTITLELOGFONT = 31
Public Const SPI_GETICONTITLEWRAP = 25
Public Const SPI_GETKEYBOARDDELAY = 22
Public Const SPI_GETKEYBOARDPREF = 68
Public Const SPI_GETKEYBOARDSPEED = 10
Public Const SPI_GETLOWPOWERACTIVE = 83
Public Const SPI_GETLOWPOWERTIMEOUT = 79
Public Const SPI_GETMENUDROPALIGNMENT = 27
Public Const SPI_GETMINIMIZEDMETRICS = 43
Public Const SPI_GETMOUSE = 3
Public Const SPI_GETMOUSEKEYS = 54
Public Const SPI_GETMOUSETRAILS = 94
Public Const SPI_GETNONCLIENTMETRICS = 41
Public Const SPI_GETPOWEROFFACTIVE = 84
Public Const SPI_GETPOWEROFFTIMEOUT = 80
Public Const SPI_GETSCREENREADER = 70
Public Const SPI_GETSCREENSAVEACTIVE = 16
Public Const SPI_GETSCREENSAVETIMEOUT = 14
Public Const SPI_GETSERIALKEYS = 62
Public Const SPI_GETSHOWSOUNDS = 56
Public Const SPI_GETSOUNDSENTRY = 64
Public Const SPI_GETSTICKYKEYS = 58
Public Const SPI_GETTOGGLEKEYS = 52
Public Const SPI_GETWINDOWSEXTENSION = 92
Public Const SPI_GETWORKAREA = 48
Public Const SPI_ICONHORIZONTALSPACING = 13
Public Const SPI_ICONVERTICALSPACING = 24
Public Const SPI_LANGDRIVER = 12
Public Const SPI_SCREENSAVERRUNNING = 97
Public Const SPI_SETACCESSTIMEOUT = 61
Public Const SPI_SETANIMATION = 73
Public Const SPI_SETBEEP = 2
Public Const SPI_SETBORDER = 6
Public Const SPI_SETCURSORS = 87
Public Const SPI_SETDEFAULTINPUTLANG = 90
Public Const SPI_SETDESKPATTERN = 21
Public Const SPI_SETDESKWALLPAPER = 20
Public Const SPI_SETDOUBLECLICKTIME = 32
Public Const SPI_SETDOUBLECLKHEIGHT = 30
Public Const SPI_SETDOUBLECLKWIDTH = 29
Public Const SPI_SETDRAGFULLWINDOWS = 37
Public Const SPI_SETDRAGHEIGHT = 77
Public Const SPI_SETDRAGWIDTH = 76
Public Const SPI_SETFASTTASKSWITCH = 36
Public Const SPI_SETFILTERKEYS = 51
Public Const SPI_SETFONTSMOOTHING = 75
Public Const SPI_SETGRIDGRANULARITY = 19
Public Const SPI_SETHANDHELD = 78
Public Const SPI_SETHIGHCONTRAST = 67
Public Const SPI_SETICONMETRICS = 46
Public Const SPI_SETICONS = 88
Public Const SPI_SETICONTITLELOGFONT = 34
Public Const SPI_SETICONTITLEWRAP = 26
Public Const SPI_SETKEYBOARDDELAY = 23
Public Const SPI_SETKEYBOARDPREF = 69
Public Const SPI_SETKEYBOARDSPEED = 11
Public Const SPI_SETLANGTOGGLE = 91
Public Const SPI_SETLOWPOWERACTIVE = 85
Public Const SPI_SETLOWPOWERTIMEOUT = 81
Public Const SPI_SETMENUDROPALIGNMENT = 28
Public Const SPI_SETMINIMIZEDMETRICS = 44
Public Const SPI_SETMOUSE = 4
Public Const SPI_SETMOUSEBUTTONSWAP = 33
Public Const SPI_SETMOUSEKEYS = 55
Public Const SPI_SETMOUSETRAILS = 93
Public Const SPI_SETNONCLIENTMETRICS = 42
Public Const SPI_SETPENWINDOWS = 49
Public Const SPI_SETPOWEROFFACTIVE = 86
Public Const SPI_SETPOWEROFFTIMEOUT = 82
Public Const SPI_SETSCREENREADER = 71
Public Const SPI_SETSCREENSAVEACTIVE = 17
Public Const SPI_SETSCREENSAVETIMEOUT = 15
Public Const SPI_SETSERIALKEYS = 63
Public Const SPI_SETSHOWSOUNDS = 57
Public Const SPI_SETSOUNDSENTRY = 65
Public Const SPI_SETSTICKYKEYS = 59
Public Const SPI_SETTOGGLEKEYS = 53
Public Const SPI_SETWORKAREA = 47

Public Const SPIF_UPDATEINIFILE = &H1
Public Const SPIF_SENDCHANGE = &H2
Public Const SPIF_SENDWININICHANGE = &H2

Sub HighlightText()

On Error Resume Next
Screen.ActiveControl.SelStart = 0
Screen.ActiveControl.SelLength = Len(Screen.ActiveControl.Text)
    
End Sub

Sub SetKeepOnTop(thisForm As Form, ByVal OnTop As Integer)

Select Case OnTop
Case 0
    '**Set window to NOT stay on top*************************
    Call SetWindowPos(thisForm.hwnd, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE Or SWP_NOSIZE Or SWP_NOACTIVATE)
Case Else
    '**Set window to stay on top*****************************
    Call SetWindowPos(thisForm.hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE Or SWP_NOSIZE Or SWP_NOACTIVATE)
End Select
thisForm.ZOrder

End Sub

Public Sub AdjustOnScreen(thisForm As Form)

'**Get the scaling factor between system unitss and TWIPS***********
Dim XScale&
Dim YScale&
XScale& = Screen.Width / GetSystemMetrics(SM_CXSCREEN)
YScale& = Screen.Height / GetSystemMetrics(SM_CYSCREEN)

'**Get the actual screen working area (screen minus task bars)******
Dim successl As Long
Dim workAreaHeight&
Dim workAreaWidth&
Dim scrnRect As RECT
successl = SystemParametersInfo(SPI_GETWORKAREA, 0&, scrnRect, 0&)
workAreaWidth& = (scrnRect.rectRight - scrnRect.rectLeft) * XScale&
workAreaHeight& = (scrnRect.rectBottom - scrnRect.rectTop) * YScale&

'**Check for off screen to the top or left*************************
If thisForm.Top < 0 Then thisForm.Top = 0
If thisForm.Left < 0 Then thisForm.Left = 0

'**Check for off screen to the bottom or right*********************
If thisForm.Width + thisForm.Left > workAreaWidth& Then thisForm.Left = workAreaWidth& - thisForm.Width
If thisForm.Height + thisForm.Top > workAreaHeight& Then thisForm.Top = workAreaHeight& - thisForm.Height

End Sub

Public Sub PositionFormCenter(thisForm As Form)

'**Get scale factors for converting from pixels to twips************
Dim XScale&
Dim YScale&
YScale& = Screen.Height / GetSystemMetrics(SM_CYSCREEN)
XScale& = Screen.Width / GetSystemMetrics(SM_CXSCREEN)

thisForm.Top = (Screen.Height - thisForm.Height) / 2
thisForm.Left = (Screen.Width - thisForm.Width) / 2

End Sub

Sub SavePosition(thisForm As Form, section$)
'*************************************************************
'**Saves the current position and size of the form to dataFile$
'
'**Used as follows:
'  SavePosition Me, "Position"
'  Me is the form you want to save position for.
'  "Position" is the section name to save it in.
'
'**The position information is saved in the file defined in dataFile$,
'  so this must be defined on initialization.
'*************************************************************
Dim OutEntry$
Dim successl As Long

OutEntry$ = Str$(thisForm.Left)
successl = WritePrivateProfileString(section$, "left", ByVal OutEntry$, dataFile$)
OutEntry$ = Str$(thisForm.Top)
successl = WritePrivateProfileString(section$, "top", ByVal OutEntry$, dataFile$)
OutEntry$ = Str$(thisForm.Width)
successl = WritePrivateProfileString(section$, "width", ByVal OutEntry$, dataFile$)
OutEntry$ = Str$(thisForm.Height)
successl = WritePrivateProfileString(section$, "height", ByVal OutEntry$, dataFile$)
OutEntry$ = Str$(thisForm.WindowState)
successl = WritePrivateProfileString(section$, "windowstate", ByVal OutEntry$, dataFile$)

End Sub

Sub PositionForm(thisForm As Form, ByVal section$, ByVal WhatToDo, ByVal OnTop, ByVal ShowIt)
'***************************************************************
'**Resize or reposition a form based on information in dataFile$
'  If the position is not yet recorded, a default position is used
'  If the position is partially or fully off screen, PositionFormCenter
'    is called.
'
'**Used as follows
'  Call PositionForm(Me, "Position", WhatToDo, OnTop, ShowIt)
'
'  Me is the form to reposition
'  "Position" is the name of the section in dataFile$
'  WhatToDo is one of the following:
'    RS_RESIZE (1)- only resize the form
'    RS_REPOSITION (2) - only reposition the form
'    RS_CHANGEKEEPONTOP (4) - only change the keep on top status
'    RS_SIZEPOSITION (3) - resize and reposition
'    to use more than one of the features, OR the appropriate items
'  OnTop is the requested keep on top status
'    RS_ONTOP (-1) - keep the form on top of all not-keep-on-top windows
'    RS_NOTONTOP (0) - don't keep the form on top
'  ShowIt indicates whether to show or hide the form
'    RS_SHOW (-1) - show the form
'    RS_NOSHOW (0) - hide the form
'
'**To center a form, use PositionFormCenter
'**To make sure a form is on screen, use AdjustOnScreen
'*****************************************************************
thisForm.Hide

'See if there is a recorded position******************************
Dim positionLeft&
Dim positionTop&
Dim formWidth&
Dim formHeight&

positionLeft& = GetPrivateProfileInt(section$, "left", -1, dataFile$)
positionTop& = GetPrivateProfileInt(section$, "top", -1, dataFile$)
formWidth& = GetPrivateProfileInt(section$, "width", -1, dataFile$)
formHeight& = GetPrivateProfileInt(section$, "height", -1, dataFile$)

'**Check for recorded position*************************************
If (formWidth& < 0) Then
    '**No recorded position. Position the form in the center of the screen.
    Call PositionFormCenter(thisForm)
    If OnTop <> 0 Then
        Call SetKeepOnTop(thisForm, RS_ONTOP)
    End If
    Exit Sub
End If

'**Position the form as recorded***********************************
If (WhatToDo And RS_REPOSITION) <> 0 Then
    thisForm.Left = positionLeft&
    thisForm.Top = positionTop&
End If

'**Resize the form as recorded*************************************
If (WhatToDo And RS_RESIZE) <> 0 Then
    thisForm.Width = formWidth&
    thisForm.Height = formHeight&
End If

'**Adjust back on screen if part of the form is off screen********
Call AdjustOnScreen(thisForm)

'**Show the form**************************************************
If ShowIt = True Then thisForm.Show
If (WhatToDo And RS_CHANGEKEEPONTOP) <> 0 Then SetKeepOnTop thisForm, OnTop

End Sub
Public Sub PositionToWindow(thisForm As Form, ByVal section$, ByVal WhatToDo, ByVal OnTop, ByVal ShowIt, WindowName As String)

thisForm.Hide

'**Recall memorized position if it's available*********************
Dim formWidth As Integer
formWidth = GetPrivateProfileInt(section$, "width", -1, dataFile$)

If formWidth >= 0 Then
    '**There is a recorded position, we should use it.
    Call PositionForm(thisForm, section$, WhatToDo, OnTop, ShowIt)
    Exit Sub
End If

'**Find the window handle for the reference window*****************
Dim wHandle As Long
wHandle = FindWindow(ByVal 0&, ByVal WindowName)

'**Exit if required window not found*******************************
If wHandle = 0 Then
    Call PositionForm(thisForm, section$, WhatToDo, OnTop, ShowIt)
    Exit Sub
End If

'**Find the window placement for the reference window**************
Dim successl As Long
Dim WindowPlace As WINDOWPLACEMENT
WindowPlace.wLength = 44
successl = GetWindowPlacement(wHandle, WindowPlace)

'*Exit if window placement not returned****************************
If successl = 0 Then
    Call PositionForm(thisForm, section$, WhatToDo, OnTop, ShowIt)
    Exit Sub
End If

'**Get the scaling values for this monitor*************************
Dim XScale&
Dim YScale&
XScale& = Screen.Width / GetSystemMetrics(SM_CXSCREEN)
YScale& = Screen.Height / GetSystemMetrics(SM_CYSCREEN)

'**Position the form to the right of the reference window**********
thisForm.Left = WindowPlace.normalRight * XScale&
thisForm.Top = (WindowPlace.normalTop + ((WindowPlace.normalBottom - WindowPlace.normalTop) / 8)) * YScale&
    
'**Adjust back on screen if part of the form is off screen*********
Call AdjustOnScreen(thisForm)

'**Show the window*************************************************
If ShowIt = True Then thisForm.Show
If (WhatToDo And RS_CHANGEKEEPONTOP) <> 0 Then SetKeepOnTop thisForm, OnTop

End Sub

Public Sub PositionTCToWindow()
'******************************************************************
'**This routine positions the secondary Help window used by the
'**training card instance of Help to the left hand edge of the
'**window that called training cards. If there is not enough room
'**to the right of the calling window, the Help window is positioned
'**to the left.
'******************************************************************

'**Find the window placement for the reference window**************
Dim successl As Long
Dim WindowPlace As WINDOWPLACEMENT
WindowPlace.wLength = 44
successl = GetWindowPlacement(currentTCCallingWindow.hwnd, WindowPlace)

'*Exit if window placement not returned****************************
If successl = 0 Then
    Exit Sub
End If

Dim refLeft&
Dim refRight&
Dim refTop&
Dim refBottom&
Dim XScale!
Dim YScale!
Dim helpTop&
Dim helpLeft&
Dim helpHeight&
Dim helpWidth&

'**Find the location and size of the calling window****************
refLeft& = WindowPlace.normalLeft
refRight& = WindowPlace.normalRight
refTop& = WindowPlace.normalTop
refBottom& = WindowPlace.normalBottom

'**Get the scaling values for this monitor*************************
XScale! = 1024 / GetSystemMetrics(SM_CXSCREEN)
YScale! = 1024 / GetSystemMetrics(SM_CYSCREEN)

'**Set the desired location and size of the Help window************
helpTop& = CLng(refTop& * YScale! + 75)
helpLeft& = CLng(refRight& * XScale!)
helpHeight& = 150
helpWidth& = 250

'**If not enough room to the left of the calling window, place the
'**Help window to the right.
If helpLeft& + helpWidth& > 1024 Then
    helpLeft& = refLeft& - helpWidth&
End If
If helpLeft& < 0 Then helpLeft& = 0

'**If not enough room to the right of the calling window, make sure
'**Help window is at least all the way on screen.
If helpTop& + helpHeight& > 1024 Then
    helpTop& = 1024 - helpHeight&
End If
If helpTop& < 0 Then helpTop& = 0

Dim HelpCommand$
HelpCommand$ = "PW(" & Str$(helpLeft&) & "," & Str$(helpTop&) & "," & Str$(helpWidth&) & "," & Str$(helpHeight&) & ", 1, `training')"
successl = WinHelp(currentTCCallingWindow.hwnd, GetHelpFile$("Training Cards"), HELP_TCARD Or HELP_COMMAND, ByVal HelpCommand$)

End Sub
